' Add references to:
' System.Workflow.Runtime
' System.Workflow.Activities
' System.Workflow.ComponentModel

Imports System.Workflow.Runtime
Imports System.Workflow.Runtime.Hosting
Imports System.Workflow.Activities

Public Class Form1
    Implements IBugReportService

    Private WithEvents m_WorkflowRuntime As WorkflowRuntime
    Private m_ExchangeService As ExternalDataExchangeService
    Private m_WorkflowInstance As WorkflowInstance

    ' Prepare the workflow runtime engine.
    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        m_WorkflowRuntime = New WorkflowRuntime()
        m_ExchangeService = New ExternalDataExchangeService()
        m_WorkflowRuntime.AddService(m_ExchangeService)
        m_ExchangeService.AddService(Me)
        m_WorkflowRuntime.StartRuntime()
    End Sub

    ' Start the workflow.
    Private Sub btnSubmit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSubmit.Click
        ' Get the workflow's type.
        Dim wf_type As Type = GetType(BugReportWF)

        ' Construct workflow arguments.
        Dim args As New Dictionary(Of String, Object)()
        args.Add("BugDescription", txtBugDescription.Text)

        ' Create and start the workflow.
        m_WorkflowInstance = _
            m_WorkflowRuntime.CreateWorkflow(wf_type, args)
        m_WorkflowInstance.Start()
    End Sub

    ' The workflow is done.
    Private Sub m_WorkflowRuntime_WorkflowCompleted(ByVal sender As Object, ByVal e As System.Workflow.Runtime.WorkflowCompletedEventArgs) Handles m_WorkflowRuntime.WorkflowCompleted
        Dim txt As String = "Workflow completed" & vbCrLf & vbCrLf
        txt &= "Description: " & e.OutputParameters("BugDescription").ToString() & vbCrLf
        txt &= "Assigned to: " & e.OutputParameters("AssignedEngineer").ToString() & vbCrLf
        txt &= "Result: " & e.OutputParameters("Status").ToString() & vbCrLf

        MessageBox.Show(txt)
    End Sub

#Region "IBugReportService code"
    Public Event EngineerAssigned(ByVal sender As Object, ByVal e As StringArgs) Implements IBugReportService.EngineerAssigned
    Public Event BugRejected(ByVal sender As Object, ByVal e As System.Workflow.Activities.ExternalDataEventArgs) Implements IBugReportService.BugRejected
    Public Event BugWorked(ByVal sender As Object, ByVal e As System.Workflow.Activities.ExternalDataEventArgs) Implements IBugReportService.BugWorked
    Public Event TestFailed(ByVal sender As Object, ByVal e As System.Workflow.Activities.ExternalDataEventArgs) Implements IBugReportService.TestFailed
    Public Event TestPassed(ByVal sender As Object, ByVal e As System.Workflow.Activities.ExternalDataEventArgs) Implements IBugReportService.TestPassed

    Public Delegate Sub AssignEngineerDelegate(ByVal bug_description As String)
    Public Sub AssignEngineer(ByVal bug_description As String) Implements IBugReportService.AssignEngineer
        If Me.InvokeRequired Then
            Me.Invoke(New AssignEngineerDelegate _
                (AddressOf Me.AssignEngineer), New Object() {bug_description})
        Else
            Dim engineer_name As String = _
                InputBox("Who do you want to assign to work bug report:" & _
                    vbCrLf & vbCrLf & bug_description, _
                    "Assign Engineer", "Debugger Dan")

            ' Tell the workflow who we assigned.
            Dim args As New StringArgs( _
                m_WorkflowInstance.InstanceId, engineer_name)
            RaiseEvent EngineerAssigned(Nothing, args)
        End If
    End Sub

    ' Fix the bug.
    Public Delegate Sub WorkBugDelegate(ByVal bug_description As String)
    Public Sub WorkBug(ByVal bug_description As String) Implements IBugReportService.WorkBug
        If Me.InvokeRequired Then
            Me.Invoke(New WorkBugDelegate _
                (AddressOf Me.WorkBug), New Object() {bug_description})
        Else
            ' Tell the workflow whether we fixed the bug.
            If MessageBox.Show("Did you fix the bug:" & _
                vbCrLf & vbCrLf & bug_description, _
                "Bug Fixed?", MessageBoxButtons.YesNo, _
                MessageBoxIcon.Question) _
                    = Windows.Forms.DialogResult.Yes _
            Then
                RaiseEvent BugWorked(Nothing, New ExternalDataEventArgs(m_WorkflowInstance.InstanceId))
            Else
                RaiseEvent BugRejected(Nothing, New ExternalDataEventArgs(m_WorkflowInstance.InstanceId))
            End If
        End If
    End Sub

    ' Test the bug fix.
    Public Delegate Sub TestBugDelegate(ByVal bug_description As String)
    Public Sub TestBug(ByVal bug_description As String) Implements IBugReportService.TestBug
        If Me.InvokeRequired Then
            Me.Invoke(New TestBugDelegate _
                (AddressOf Me.TestBug), New Object() {bug_description})
        Else
            ' Tell the Bugflow whether we fixed the bug.
            If MessageBox.Show("Did the test pass for:" & _
                vbCrLf & vbCrLf & bug_description, _
                "Test Passed?", MessageBoxButtons.YesNo, _
                MessageBoxIcon.Question) _
                    = Windows.Forms.DialogResult.Yes _
            Then
                RaiseEvent TestPassed(Nothing, New ExternalDataEventArgs(m_WorkflowInstance.InstanceId))
            Else
                RaiseEvent TestFailed(Nothing, New ExternalDataEventArgs(m_WorkflowInstance.InstanceId))
            End If
        End If
    End Sub

#End Region ' IBugReportService

End Class
